//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using MindFusion.Diagramming.Export;


namespace MindFusion.Diagramming.WinForms.Samples.CS.TabbedView
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			DiagramStyle style = new DiagramStyle();
			style.BackBrush = new MindFusion.Drawing.SolidBrush(Color.White);
			style.ShadowBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(20, Color.Black));
			style.NodeEffects.Add(new SmoothShadowEffect());

			ShapeNodeStyle nodeStyle = new ShapeNodeStyle();
			nodeStyle.Brush = new MindFusion.Drawing.LinearGradientBrush(
				Color.White, Color.FromArgb(219, 231, 240), 90);

			DiagramLinkStyle linkStyle = new DiagramLinkStyle();
			linkStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.Black);

			theme = new Theme();
			theme.RegisterStyle(typeof(Diagram), style);
			theme.RegisterStyle(typeof(ShapeNode), nodeStyle);
			theme.RegisterStyle(typeof(DiagramLink), linkStyle);

			linkShape = new Shape(
				new ElementTemplate[]
				{
					new LineTemplate(20, 100, 50, 0),
					new LineTemplate(50, 0, 80, 100),
					new LineTemplate(80, 100, 20, 100)
				},
				FillMode.Winding, "myShape");


			SetupPage(tabbedDiagramView.CurrentPage);
		}

		private void tabbedDiagramView_PageAdded(object sender, DiagramPageEventArgs e)
		{
			SetupPage(e.DiagramPage);
		}

		private void tabbedDiagramView_PageRemoving(object sender, DiagramPageValidationEventArgs e)
		{
			if (tabbedDiagramView.Document.Pages.Count == 1)
			{
				MessageBox.Show(this, "The last page cannot be removed.", "Removing page",
					MessageBoxButtons.OK, MessageBoxIcon.Warning);
				e.Cancel = true;
			}
		}

		private void SetupPage(DiagramPage page)
		{
			page.EffectsProvider = new EffectsProvider();
			page.Theme = theme;
			page.ShowGrid = true;
			page.GridStyle = GridStyle.Points;
			page.DefaultShape = Shapes.Rectangle;
			page.RouteLinks = true;
			page.RoundedLinks = true;
			page.RoundedLinksRadius = 5;
			page.LinkHeadShape = linkShape;
			page.LinkHeadShapeSize = 3;
			page.ShadowOffsetX = 2;
			page.ShadowOffsetY = 2;
		}

		private void OnShowTabsChanged(object sender, EventArgs e)
		{
			tabbedDiagramView.ShowTabs = !radioNever.Checked;
			if (!radioNever.Checked)
			{
				if (radioLeft.Checked)
					tabbedDiagramView.TabAlignment = TabAlignment.Left;
				else if (radioTop.Checked)
					tabbedDiagramView.TabAlignment = TabAlignment.Top;
				else if (radioRight.Checked)
					tabbedDiagramView.TabAlignment = TabAlignment.Right;
				else if (radioBottom.Checked)
					tabbedDiagramView.TabAlignment = TabAlignment.Bottom;
			}
		}

		private void checkAllowReorder_CheckedChanged(object sender, EventArgs e)
		{
			tabbedDiagramView.AllowTabReorder = checkAllowReorder.Checked;
		}

		private void checkShowCreateNewButton_CheckedChanged(object sender, EventArgs e)
		{
			tabbedDiagramView.ShowCreateNewTab = checkShowCreateNewButton.Checked;
		}

		private void checkShowCloseButtons_CheckedChanged(object sender, EventArgs e)
		{
			tabbedDiagramView.ShowTabCloseButtons = checkShowCloseButtons.Checked;
		}

		private void checkShowNavigationButtons_CheckedChanged(object sender, EventArgs e)
		{
			tabbedDiagramView.ShowTabNavigationButtons = checkShowNavigationButtons.Checked;
		}

		private void openToolStripMenuItem_Click(object sender, EventArgs e)
		{
			OpenFileDialog dialog = new OpenFileDialog();
			dialog.Filter = "XML Files|*.xml|All Files|*.*||";
			if (dialog.ShowDialog(this) == DialogResult.OK)
			{
				tabbedDiagramView.LoadFromXml(dialog.FileName);

				foreach (DiagramPage page in tabbedDiagramView.Document.Pages)
					SetupPage(page);
			}
		}

		private void saveToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog dialog = new SaveFileDialog();
			dialog.Filter = "XML Files|*.xml|All Files|*.*||";
			if (dialog.ShowDialog(this) == DialogResult.OK)
				tabbedDiagramView.SaveToXml(dialog.FileName);
		}

		private void printPreviewToolStripMenuItem_Click(object sender, EventArgs e)
		{
			tabbedDiagramView.PrintPreviewEx();
		}

		private void printToolStripMenuItem_Click(object sender, EventArgs e)
		{
			tabbedDiagramView.PrintOptions.FromDiagramPage = null;
			tabbedDiagramView.PrintOptions.ToDiagramPage = null;
			tabbedDiagramView.Print();
		}

		private void printCurrentDiagramToolStripMenuItem_Click(object sender, EventArgs e)
		{
			int pageIndex = tabbedDiagramView.Document.Pages.IndexOf(tabbedDiagramView.CurrentPage);
			tabbedDiagramView.PrintOptions.FromDiagramPage = pageIndex;
			tabbedDiagramView.PrintOptions.ToDiagramPage = pageIndex;
			tabbedDiagramView.Print();
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void pDFToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog dialog = new SaveFileDialog();
			dialog.Filter = "PDF Files|*.pdf|All Files|*.*||";
			if (dialog.ShowDialog(this) == DialogResult.OK)
			{
				PdfExporter exporter = new PdfExporter();
				exporter.Export(tabbedDiagramView.Document, dialog.FileName);
			}
		}

		private void visioToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SaveFileDialog dialog = new SaveFileDialog();
			dialog.Filter = "Visio Files|*.vdx|All Files|*.*||";
			if (dialog.ShowDialog(this) == DialogResult.OK)
			{
				VisioExporter exporter = new VisioExporter();
				exporter.Export(tabbedDiagramView.Document, dialog.FileName);
			}
		}


		private Theme theme;
		private Shape linkShape;
	}
}
